#include "StdAfx.h"
#include <shlobj.h>
#include "registry.h"
#include "util.h"
#include "ttf.h"

extern HWND hWndProgressBar;
extern HWND hWndFontName;
extern HWND hWndMessage;

static BOOL GetFontFolder( char* folder );
static void AddFont( const char* ttf_folder, const char* ttf_name );
void RegistTrueTypeFonts( const char* ttf_folder );

BOOL IsInstalledWinTeX( void )
{
	char font_path[MAX_PATH];
	if ( GetFontFolder(font_path) == FALSE ) return FALSE;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts", "r" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts", "r");
	if( rp == NULL ) return FALSE;
	char check_name[128];
	wsprintf( check_name, "%s Regular (TrueType)", WinTeX0[0] );
	char ttf_filename[MAX_PATH];
	const char* val = rgets( ttf_filename, MAX_PATH, rp, check_name );
	rclose( rp );
	if( val == NULL ) return FALSE;
	else return TRUE;
}

BOOL IsInstalledWinTeXAmS( void )
{
	char font_path[MAX_PATH];
	if ( GetFontFolder(font_path) == FALSE ) return FALSE;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts", "r" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts", "r" );
	if( rp == NULL ) return FALSE;
	char check_name[128];
	wsprintf( check_name, "%s Regular (TrueType)", WinTeX1[67] );
	char ttf_filename[MAX_PATH];
	const char* val = rgets( ttf_filename, MAX_PATH, rp, check_name );
	rclose( rp );
	if( val == NULL ) return FALSE;
	else return TRUE;
}

BOOL IsInstalledBaKoMa( void )
{
	char font_path[MAX_PATH];
	if ( GetFontFolder(font_path) == FALSE ) return FALSE;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts", "r" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts", "r" );
	if( rp == NULL ) return FALSE;
	char check_name[128];
	wsprintf( check_name, "%s (TrueType)", BaKoMa[87] );
	char ttf_filename[MAX_PATH];
	const char* val = rgets( ttf_filename, MAX_PATH, rp, check_name );
	rclose( rp );
	if( val == NULL ) return FALSE;
	else return TRUE;
}

void UninstallBaKoMa( void )
{
	if( IsInstalledBaKoMa() == FALSE ) return;
	char ttf_folder[MAX_PATH];
	if ( GetFontFolder(ttf_folder) == FALSE ) return;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts" );
	if( rp == NULL )
	{
		MessageBox( NULL, "ACXg[sł܂B\n"
			"sɕKvȌȂ̂܂B",
			"BaKoMa installer", MB_OK );
		return;
	}

	SendMessage( hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0,139) );
	SendMessage( hWndProgressBar, PBM_SETSTEP, 1, 0 );

	for( int i = 0; i <= 139; i++ )
	{
		char reg_ttf_name[MAX_PATH];
		wsprintf( reg_ttf_name, "%s (TrueType)", BaKoMa[i] );
		char ttf_filename[MAX_PATH];
		SetWindowText( hWndFontName, BaKoMa[i] );
		if( rgets(ttf_filename, MAX_PATH, rp, reg_ttf_name) == NULL ) continue;
		rdelete_value( rp, reg_ttf_name );
		RemoveFontResource( reg_ttf_name );
		if( FindFirstChar(ttf_filename, '\\') == -1 )
		{ // ttf_filename doesn't include '\'
			if( FindFirstChar(ttf_filename, '.') == -1 ) lstrcat( ttf_filename, ".ttf" );
			int l = lstrlen( ttf_folder );
			if( l == 0 ) continue;
			char ttfname[MAX_PATH];
			if( ttf_folder[l-1] == '\\' )
				wsprintf( ttfname, "%s%s", ttf_folder, ttf_filename );
			else
				wsprintf( ttfname, "%s\\%s", ttf_folder, ttf_filename );
			SetFileAttributes( ttfname, FILE_ATTRIBUTE_NORMAL );
			DeleteFile( ttfname );
		}
		SendMessage( HWND_BROADCAST, WM_FONTCHANGE, 0, 0 );
		SendMessage( hWndProgressBar, PBM_STEPIT, 0, 0 );
	}
	rclose( rp );
	return;
}

void UninstallWinTeX( void )
{
	if( IsInstalledWinTeX() == FALSE ) return;
	char ttf_folder[MAX_PATH];
	if ( GetFontFolder(ttf_folder) == FALSE ) return;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts" );
	if( rp == NULL )
	{
		MessageBox( NULL, "ACXg[sł܂B\n"
			"sɕKvȌȂ̂܂B",
			"BaKoMa installer", MB_OK );
		return;
	}

	SendMessage( hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0,64) );
	SendMessage( hWndProgressBar, PBM_SETSTEP, 1, 0 );

	for( int i = 0; i <= 64; i++ )
	{
		char reg_ttf_name[MAX_PATH];
		wsprintf( reg_ttf_name, "%s Regular (TrueType)", WinTeX0[i] );
		SetWindowText( hWndFontName, WinTeX0[i] );
		char ttf_filename[MAX_PATH];
		if( rgets(ttf_filename, MAX_PATH, rp, reg_ttf_name) == NULL ) continue;
		rdelete_value( rp, reg_ttf_name );
		RemoveFontResource( reg_ttf_name );
		if( FindFirstChar(ttf_filename, '\\') == -1 )
		{ // ttf_filename doesn't include '\'
			if( FindFirstChar(ttf_filename, '.') == -1 ) lstrcat( ttf_filename, ".ttf" );
			int l = lstrlen( ttf_folder );
			if( l == 0 ) continue;
			char ttfname[MAX_PATH];
			if( ttf_folder[l-1] == '\\' )
				wsprintf( ttfname, "%s%s", ttf_folder, ttf_filename );
			else
				wsprintf( ttfname, "%s\\%s", ttf_folder, ttf_filename );
			SetFileAttributes( ttfname, FILE_ATTRIBUTE_NORMAL );
			DeleteFile( ttfname );
		}
		SendMessage( HWND_BROADCAST, WM_FONTCHANGE, 0, 0 );
		SendMessage( hWndProgressBar, PBM_STEPIT, 0, 0 );
	}
	rclose( rp );
	return;
}

void UninstallWinTeXAmS( void )
{
	if( IsInstalledWinTeXAmS() == FALSE ) return;
	char ttf_folder[MAX_PATH];
	if ( GetFontFolder(ttf_folder) == FALSE ) return;
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts" );
	if( rp == NULL )
	{
		MessageBox( NULL, "ACXg[sł܂B\n"
			"sɕKvȌȂ̂܂B",
			"BaKoMa installer", MB_OK );
		return;
	}

	SendMessage( hWndProgressBar, PBM_SETPOS, 0, 0 );
	SendMessage( hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0,64) );
	SendMessage( hWndProgressBar, PBM_SETSTEP, 1, 0 );
	SetWindowText( hWndMessage, "AMStHg̃ACXg[" );

	for( int i = 0; i <= 88; i++ )
	{
		char reg_ttf_name[MAX_PATH];
		wsprintf( reg_ttf_name, "%s Regular (TrueType)", WinTeX1[i] );
		SetWindowText( hWndFontName, WinTeX1[i] );
		char ttf_filename[MAX_PATH];
		if( rgets(ttf_filename, MAX_PATH, rp, reg_ttf_name) == NULL ) continue;
		rdelete_value( rp, reg_ttf_name );
		RemoveFontResource( reg_ttf_name );
		if( FindFirstChar(ttf_filename, '\\') == -1 )
		{ // ttf_filename doesn't include '\'
			if( FindFirstChar(ttf_filename, '.') == -1 ) lstrcat( ttf_filename, ".ttf" );
			int l = lstrlen( ttf_folder );
			if( l == 0 ) continue;
			char ttfname[MAX_PATH];
			if( ttf_folder[l-1] == '\\' )
				wsprintf( ttfname, "%s%s", ttf_folder, ttf_filename );
			else
				wsprintf( ttfname, "%s\\%s", ttf_folder, ttf_filename );
			SetFileAttributes( ttfname, FILE_ATTRIBUTE_NORMAL );
			DeleteFile( ttfname );
		}
		SendMessage( HWND_BROADCAST, WM_FONTCHANGE, 0, 0 );
		SendMessage( hWndProgressBar, PBM_STEPIT, 0, 0 );
	}
	rclose( rp );
	return;
}

void InstallBaKoMa( void )
{
	char ttf_dir[MAX_PATH];
	ttf_dir[0] = NULL;
	
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts" );
	if( rp == NULL )
	{
		MessageBox( NULL, "CXg[sł܂B\n"
			"sɕKvȌȂ̂܂B",
			"BaKoMa installer", MB_OK );
		return;
	}
	rclose( rp );

	char drive_name[4];
	wsprintf( drive_name, "a:\\" );
search_cd:;
	for( char c = 'z'; c >= 'a'; c-- )
	{
		drive_name[0] = c;
		if( GetDriveType(drive_name) == 1 ) continue;
		HANDLE hFind;
		WIN32_FIND_DATA ffd;
		char search_file[MAX_PATH];
		wsprintf( search_file, "%sCM-TrueType", drive_name );
		hFind = FindFirstFile( search_file, &ffd );
		if( hFind == INVALID_HANDLE_VALUE )
			continue;
		else
		{
			FindClose( hFind );
			wsprintf( ttf_dir, "%sCM-TrueType", drive_name );
			break;
		}
	}
	if( ttf_dir[0] == NULL )
	{
		int r = MessageBox( NULL, "pLaTeX2e for Windows Another Manual Vol.1 BasicKit 1999\n"
					"CD-ROM}ĂB", "pLaTeX2e for Win SP1",
					MB_OKCANCEL );
		if( r == IDCANCEL ) return;
		goto search_cd;
	}
	RegistTrueTypeFonts( ttf_dir );
	return;
}

static BOOL GetFontFolder( char* folder )
{
	LPITEMIDLIST ppidl; // PIDL (Pointer to ID List)
	HRESULT hr;

	hr = SHGetSpecialFolderLocation( NULL, CSIDL_FONTS, &ppidl );
	if( FAILED(hr) ) return FALSE;
	return SHGetPathFromIDList( ppidl, folder );
}

void RegistTrueTypeFonts( const char* ttf_folder )
{
	char ttfname[MAX_PATH];

	int l = lstrlen( ttf_folder );
	if( l == 0 ) return;
	if( ttf_folder[l-1] == '\\' )
		wsprintf( ttfname, "%s*.ttf", ttf_folder );
	else
		wsprintf( ttfname, "%s\\*.ttf", ttf_folder );
	
	SendMessage( hWndProgressBar, PBM_SETRANGE, 0, MAKELPARAM(0,139) );
	SendMessage( hWndProgressBar, PBM_SETSTEP, 1, 0 );
	WIN32_FIND_DATA ffd; 
	HANDLE hFind = FindFirstFile( ttfname, &ffd );
	if( hFind == INVALID_HANDLE_VALUE ) return;
	do
	{
		SetWindowText( hWndFontName, ffd.cFileName );
		AddFont( ttf_folder, ffd.cFileName );
		SendMessage( hWndProgressBar, PBM_STEPIT, 0, 0 );
	} while( FindNextFile(hFind,&ffd) != FALSE );
	FindClose( hFind );
}

static void AddFont( const char* ttf_folder, const char* ttf_name )
{
	int len = lstrlen(ttf_folder) + lstrlen(ttf_name) + 2;
	
	char* ttf_fullpath = (char*)malloc( len*sizeof(char) );

	int l = lstrlen( ttf_folder );
	if( l == 0 ) return;
	if( ttf_folder[l-1] == '\\' )
		wsprintf( ttf_fullpath, "%s%s", ttf_folder, ttf_name );
	else
		wsprintf( ttf_fullpath, "%s\\%s", ttf_folder, ttf_name );

	// Copying TrueType Fonts to the system font directory. 
	char font_folder[MAX_PATH];
	if( GetFontFolder(font_folder) == FALSE ) return;
	lstrcat( font_folder, "\\" );
	lstrcat( font_folder, ttf_name );
	if( CopyFile(ttf_fullpath, font_folder, TRUE) == FALSE ) return;

	// regist ttf in the system font directory.
	int r = AddFontResource( font_folder );
	REG* rp;
	if( IsWindowsNT() )
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows NT\\CurrentVersion\\Fonts" );
	else
		rp = ropen( HKEY_LOCAL_MACHINE, "Software\\Microsoft\\Windows\\CurrentVersion\\Fonts" );
	// Creating fonts name.
	char font_reg_key[128];
	char font_reg_node[128];
	lstrcpy( font_reg_node, ttf_name );
	int p = FindLastChar( font_reg_node, '.' );
	if( p != 0 )
	{
		font_reg_node[p] = NULL;
	}
	wsprintf( font_reg_key, "%s (TrueType)", font_reg_node );
	lstrcat( font_reg_node, ".ttf" );

	if( r != 0 )
	{
		rputs( font_reg_node, rp, font_reg_key );
		SendMessage( HWND_BROADCAST, WM_FONTCHANGE, 0, 0 );
	}
	rclose( rp );
	free( ttf_fullpath );
	return;
}

BOOL IsWindowsNT( void )
{
    OSVERSIONINFO osversion;
    memset( &osversion, 0, sizeof(OSVERSIONINFO) );
    osversion.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    GetVersionEx(&osversion);
    if( osversion.dwPlatformId == VER_PLATFORM_WIN32_NT )
    {
        return TRUE;
    }
    return FALSE;
}

int FindLastChar( const char* str, char c )
{
	/* This function checks whether the char c exists in the str.
	   If c exists, this function returns the LAST index of c.
	   If not exists, return -1; */
	int ret = -1;
	BOOL prev_is_kanji1 = FALSE;
	int l = lstrlen( str );
	for( int i = 0; i < l; i ++ )
	{
		if( prev_is_kanji1 == TRUE )
		{// we must skip the 2nd byte of Shift-JIS.
		 // the next character of 2nd byte of Shift-JIS is always not a 2nd byte of Kanji.
			prev_is_kanji1 = FALSE;
			continue;
		}
		else // prev_is_kanji == FALSE
		{// the previous character is not Shift-JIS first code. 
			if( str[i] == c )
			{
				ret = i;
				continue;
			}
			else if( IsKanji1(str[i]) == TRUE )
			{
				prev_is_kanji1 = TRUE;
				continue;
			}
			else
				continue;
		}
	}
	return ret;
}

int FindFirstChar( const char* str, char c )
{
	/* This function checks whether the char c exists in the str.
	   If c exists, this function returns the FIRST index of c.
	   If not exists, return -1; */
	BOOL prev_is_kanji1 = FALSE;
	int l = lstrlen( str );
	for( int i = 0; i < l; i ++ )
	{
		if( prev_is_kanji1 == TRUE )
		{// we must skip the 2nd byte of Shift-JIS.
		 // the next character of 2nd byte of Shift-JIS is always not a 2nd byte of Kanji.
			prev_is_kanji1 = FALSE;
			continue;
		}
		else // prev_is_kanji == FALSE
		{// the previous character is not Shift-JIS first code. 
			if( str[i] == c )
			{
				return i;
				continue;
			}
			else if( IsKanji1(str[i]) == TRUE )
			{
				prev_is_kanji1 = TRUE;
				continue;
			}
			else
				continue;
		}
	}
	return -1;
}

BOOL IsKanji1( const unsigned char c )
{
	if( (c >= 0x81 && c <= 0x9F) || (c >= 0xE0 && c <= 0xFC) )
		return TRUE;
	else
		return FALSE;
}
